const path = require('path');
const chalk = require('chalk');
const paths = require('./paths');
const fs = require('fs');

const escapeRegExp = (reStr) => reStr.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
const currPathPrefix = escapeRegExp(path.join(paths.appPath, 'e2e'));
const stackLineRegExp = new RegExp(`^.*(${currPathPrefix}[^:]+):(\\d+):(\\d+).*$`);

module.exports = {
  internalStackTrace: chalk.yellow('  at somewhere in "selenuim-webdriver"'),
  splitStackTrace: (stackTrace) => {
    const allTraceLines = stackTrace.split('\n').map((line) => line.trim());
    const lastTraceIndex = allTraceLines.findIndex((line) => line === 'From asynchronous test:');

    return lastTraceIndex < 0 ? allTraceLines : allTraceLines.slice(0, lastTraceIndex);
  },
  processStackTraceLine: (stackTraceLine) => {
    const matches = stackTraceLine.match(stackLineRegExp);
    if (!matches) {
      return null;
    }

    const [, file, line, pos] = matches;
    return chalk.yellow(`  at ${path.relative(paths.appPath, file)}:${line}:${pos}`);
  },
  getSpecFilesPattern: (suite = undefined) => {
    if (!suite) {
      return [
        path.join(paths.appPath, 'e2e/**/*.spec.ts'),
        path.join(paths.appPath, 'e2e/**/*.spec.js'),
      ];
    }

    if (!fs.existsSync(path.join(paths.appPath, 'e2e/' + suite))) {
      throw new Error('Invalid "suite" argument value');
    }

    return fs.lstatSync(path.join(paths.appPath, `e2e/${suite}`)).isDirectory()
        ? [
          path.join(paths.appPath, `e2e/${suite}/*.spec.js`),
          path.join(paths.appPath, `e2e/${suite}/*.spec.ts`),
        ]
        : [
          path.join(paths.appPath, `e2e/${suite}`),
        ];
  },
};
